# coding=utf-8
#
#   Copyright 2013-2014 Thadeu Lima de Souza Cascardo <cascardo@cascardo.info>
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
# -*- mode: python; encoding: utf-8; -*-
import xml.dom.minidom
import dirs
import os
import form
import ocupacoes

class OcupacaoForm(form.OptionsForm):
    def __init__(self, ocupacoes, contribuinte):
        g = ocupacoes.groups()
        l = []
        for i in sorted(g):
            l.extend(g[i])
        o = map(lambda x: (x[0], x[3]), l)
        form.OptionsForm.__init__(self, u"Ocupações", o, contribuinte.get_campo_contribuinte("ocupacaoPrincipal"))
        self.ocupacoes = ocupacoes
        self.contribuinte = contribuinte
    def set_value(self, value):
        form.OptionsForm.set_value(self, value)
        self.contribuinte.set_campo_contribuinte("ocupacaoPrincipal", value)

class ContribuinteForm(form.StringForm):
    def __init__(self, name, attr, contribuinte):
        self.contribuinte = contribuinte
        self.attr = attr
        form.StringForm.__init__(self, name, self.contribuinte.get_campo_contribuinte(self.attr))
    def set_value(self, value):
        form.StringForm.set_value(self, value)
        self.contribuinte.set_campo_contribuinte(self.attr, value)

class Contribuinte:
    def __init__(self, cpf):
        irpf_dir = dirs.get_default_irpf_dir()
        self.cpf = self._minimize_cpf(cpf)

        if not self._validate_cpf(self.cpf):
            raise RuntimeError("Invalid CPF: " + self.cpf)

        if not os.path.exists(irpf_dir.get_resource_dir()):
            raise RuntimeError("O caminho para o resource não existe: " + \
                    irpf_dir.get_resource_dir())

        if not os.path.exists(irpf_dir.get_userdata_dir()):
            raise RuntimeError("O caminho para os dados não existe: " + \
                    irpf_dir.get_userdata_dir())

        self.cpf_file = irpf_dir.get_userdata_file("%s/%s.xml" % (self.cpf, self.cpf))
        self.iddecl_file = irpf_dir.get_userdata_file("iddeclaracoes.xml")
        self.declaracao = self._find_id()
        self.dados = xml.dom.minidom.parse(self.cpf_file)
        self.contribuinte = self.dados.getElementsByTagName("contribuinte")[0]

    def _find_id(self):
        cpf = self._normalize_cpf(self.cpf)
        self.declaracoes = xml.dom.minidom.parse(self.iddecl_file)
        for i in self.declaracoes.getElementsByTagName("item"):
            if "cpf" in i.attributes.keys():
                if i.attributes["cpf"].nodeValue == cpf:
                    return i
        return None

    # CPF normalizado se parece com 000.000.000-00
    def _normalize_cpf(self, cpf):
        ncpf = ""
        for i in cpf:
            if len(ncpf) == 3 or len(ncpf) == 7:
                ncpf += '.'
            if len(ncpf) == 11:
                ncpf += '-'
            if len(ncpf) == 14:
                break
            if ord(i) >= ord('0') and ord(i) <= ord('9'):
                ncpf += i
        if len(ncpf) != 14:
            raise RuntimeError("Invalid CPF")
        return ncpf

    # CPF minimizado se parece com 01234567890
    def _minimize_cpf(self, cpf):
        return self._minimize_valor(cpf)

    def _minimize_valor(self, valor):
        nvalor = ''.join(e for e in valor if e.isalnum())
        return str(nvalor)

    def _validate_cpf(self, cpf):
        if len(cpf) != 11:
            return False
        return self._validate_generico(cpf)

    def _validate_generico(self, valor):
        def calcula_digito_verificador(numero):
            n = len(numero) + 1

            soma = 0
            for i in range(n):
                if i > len(numero) - 1:
                    break
                soma = soma + int(numero[i]) * ( n - i)

            dv =  soma % 11

            if dv < 2:
                dv = 0
            else:
                dv = 11 - dv

            return numero + str(dv)

        mcpf = self._minimize_valor(valor)
        cpf_sem_dv = mcpf[:-2]

        primeiro_dv = str(calcula_digito_verificador(cpf_sem_dv))
        segundo_dv = calcula_digito_verificador(primeiro_dv)

        return segundo_dv == mcpf

    def save(self):
        self.dados.writexml(open(self.cpf_file, "w"))
        self.declaracoes.writexml(open(self.iddecl_file, "w"))

    def _get_attr(self, el, attr):
        if attr in el.attributes.keys():
            return el.attributes[attr].nodeValue
        return None

    def _set_attr(self, el, attr, val):
        el.attributes[attr].nodeValue = val

    def get_declaracao(self, attr):
        return self._get_attr(self.declaracao, attr)

    def set_declaracao(self, attr, val):
        self._set_attr(self.declaracao, attr, val)

    def get_nome(self):
        return self.get_declaracao("nome")

    def set_nome(self, nome):
        self.set_declaracao("nome", nome)

    def get_campo_contribuinte(self, attr):
        if attr == "nome":
            return self.get_nome()
        return self._get_attr(self.contribuinte, attr)

    def set_campo_contribuinte(self, attr, val):
        if attr == "nome":
            self.set_nome(val)
        else:
            self._set_attr(self.contribuinte, attr, val)

    def form(self):
        form = []
        ocup = ocupacoes.Ocupacoes()
        form.append(ContribuinteForm("Nome", "nome", self))
        form.append(OcupacaoForm(ocup, self))
        for i in contribuinte_attributes:
            form.append(ContribuinteForm(i, i, self))
        return form

contribuinte_attributes = [
        "nome",
        "dataNascimento",
        "tituloEleitor",
        "doencaDeficiencia",
        "exterior",
        "pais",
        "cep",
        "uf",
        "cidade",
        "municipio",
        "tipoLogradouro",
        "logradouro",
        "numero",
        "complemento",
        "bairro",
        "bairroExt",
        "cepExt",
        "logradouroExt",
        "numeroExt",
        "complementoExt",
        "ocupacaoPrincipal",
        "codigoExterior",
        "ddd",
        "telefone",
        "naturezaOcupacao",
        ]

declaracao_attributes = [
        "dataUltimoAcesso",
        "declaracaoRetificadora",
        "enderecoDiferente",
        "enderecoMACRede",
        "exercicio",
        "nome",
        "numReciboDecRetif",
        "numeroReciboDecAnterior",
        "resultadoDeclaracao",
        "tipoDeclaracao",
        "tipoDeclaracaoAES",
        "transmitida",
        "versaoBeta"
        ]

if __name__ == '__main__':
    import sys
    contribuinte = Contribuinte(sys.argv[1])
    print "Carregando CPF " + contribuinte._normalize_cpf(sys.argv[1])

    if len(sys.argv) == 4:
        print "Valor anterior: " + contribuinte.get_campo_contribuinte(sys.argv[2])
        contribuinte.set_campo_contribuinte(sys.argv[2], sys.argv[3])
        print "Valor atual: " + contribuinte.get_campo_contribuinte(sys.argv[2])
        print "Salvando..."
        contribuinte.save()
    elif len(sys.argv) == 3:
        campo = sys.argv[2]
        valor = contribuinte.get_campo_contribuinte(campo)
        if valor:
            print ("Valor de " + campo + ": " + valor)
        else:
            print ("Campo " + campo + " retornou vazio")
    else:
        print "\nCONTRIBUINTE:"
        for i in contribuinte_attributes:
            val = contribuinte.get_campo_contribuinte(i)
            if val == None:
                val = ""
            print i + ": " + val
        print "\nDECLARACAO:"
        for i in declaracao_attributes:
            val = contribuinte.get_declaracao(i)
            if val == None:
                val = ""
            print i + ": " + val

# vim:tabstop=4:expandtab:smartindent
